#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Created by:             Lukas E. G. Fischer
                        Methods Center and Institute of Sports Science • University of Tübingen
                        lukas.fischer@uni-tuebingen.de

Execution Time:         2s (approximately)

This script produces the figures, tables, and the regression output used in "eet_paper.pdf". Close all depending files
before running.
"""

# TODO Add comments

# %% Modules and Functions

import logging
import pickle

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
import statsmodels.api as sm

from format_num import format_num
from write_text_file import write_text_file

# %% Log File

logging.basicConfig(filename="Log_Files/eet_main_script.log", level=logging.INFO,
                    format="%(levelname)s %(asctime)s - %(message)s", filemode="w")

# %% Cockpit

START_TIME = pd.Timestamp.now()
SEED = 0

# %% Script Setup

np.random.seed(SEED)

pd.set_option("display.max_colwidth", None)

plt.close("all")
sns.set(style="ticks")  # Sets style for Matplotlib and Seaborn plots consistently

# %% Data

ansc_dtypes = pickle.load(open("Objects/eet_ansc_dtypes_dict.pkl", "rb"))

# If dtype is not specified and the DtypeWarning pops up (column counting starts at 0), it is very likely that a string
# column contains missing values, for example
ansc_1 = pd.read_csv("../Data/anscombe_1.csv", dtype=ansc_dtypes)
ansc_2 = pd.read_csv("../Data/anscombe_2.csv", dtype=ansc_dtypes)
ansc_3 = pd.read_csv("../Data/anscombe_3.csv", dtype=ansc_dtypes)
ansc_4 = pd.read_csv("../Data/anscombe_4.csv", dtype=ansc_dtypes)

# %% Plots

ansc_list = [ansc_1, ansc_2, ansc_3, ansc_4]
i = 1
ansc_desc = pd.DataFrame()

for df in ansc_list:
    sns.regplot(x="x", y="y", data=df, ci=None, line_kws={"color": "#DD8452"})
    plt.savefig("../Figures/eet_reg_ansc_" + str(i) + ".pdf", bbox_inches="tight", transparent=True)
    plt.show()

    # %% Descriptive Statistics

    desc_temp = df.describe()
    desc_temp = desc_temp.rename(columns={"x": "x" + str(i), "y": "y" + str(i)})
    ansc_desc = pd.concat([ansc_desc, desc_temp.transpose()])
    i += 1

ansc_desc_1 = ansc_desc[["count", "mean", "std", "min"]]
ansc_desc_2 = ansc_desc[["25%", "50%", "75%", "max"]]

desc_metrics_1 = ansc_desc_1.columns.to_list()
desc_metrics_2 = ansc_desc_2.columns.to_list()

ansc_all_long = pd.concat([ansc_1, ansc_2, ansc_3, ansc_4]).reset_index()
ansc_all_x_skew = ansc_all_long["x"].skew()
ansc_all_x_kurt = ansc_all_long["x"].kurtosis()
ansc_all_x_metrics = np.array([ansc_all_x_skew, ansc_all_x_kurt])

# %% Linear Regression

idx = np.random.randint(0, 4)

x = ansc_list[idx]["x"]
y = ansc_list[idx]["y"]

lin_reg = sm.OLS(y, x).fit()

# %% TeX Files

write_text_file(file="../Tables/eet_desc_ansc_all.tex",
                body=ansc_desc.to_latex(formatters=[format_num(0, "f"), format_num(2, "f"), format_num(2, "f"), None,
                                                    format_num(2, "f"), None, None, None]),
                first_line=1, last_line=-1)

write_text_file(file="../Tables/eet_desc_1_ansc_all.tex",
                body=ansc_desc_1.to_latex(formatters=[format_num(4, "g"), format_num(2, "f"), format_num(2, "f"),
                                                      None]),
                first_line=1, last_line=-1)

write_text_file(file="../Tables/eet_desc_2_ansc_all.tex",
                body=ansc_desc_2.to_latex(formatters=[format_num(2, "f"), None, None, None]),
                first_line=1, last_line=-1)

write_text_file(file="../Tables/eet_ansc_all_long.tex",
                body=ansc_all_long.to_latex(index=False, formatters=[None, None, None]), first_line=1, last_line=-1)

write_text_file(file="../Tables/eet_reg_outp_ansc_1.tex", body=str(lin_reg.summary()), first_line=1, last_line=-4)

# %% Saving

pickle.dump(desc_metrics_1, open("Objects/eet_desc_metrics_1_list.pkl", "wb"))
pickle.dump(desc_metrics_2, open("Objects/eet_desc_metrics_2_list.pkl", "wb"))
ansc_all_long.to_pickle("Objects/eet_ansc_all_long_df.pkl")
np.save("Objects/eet_ansc_all_x_skew_val.npy", ansc_all_x_skew)
lin_reg.save("Models/eet_lin_reg.pkl")

# %% Execution Time and Log Finish

logging.info("Execution Time: " + str((pd.Timestamp.now() - START_TIME).round("s")))
logging.shutdown()
